@@ -1,6 +1,8 @@
 use crate::insn::MAX_CHAR_SET_LENGTH;
-use std::fmt;
+use core::fmt;
 extern crate memchr;
+#[cfg(not(feature = "std"))]
+use alloc::vec::Vec;
 
 /// Facilities for searching bytes.
 pub trait ByteSearcher {
@@ -11,69 +13,64 @@ pub trait ByteSearcher {
 }
 
 /// Helper trait that describes matching against literal bytes.
-pub trait ByteSeq: ByteSearcher + std::fmt::Debug + Copy + Clone {
+pub trait ByteSeq: core::fmt::Debug + Copy + Clone {
     /// Number of bytes.
     const LENGTH: usize;
 
-    /// Get the bytes as a slice.
-    fn as_slice(&self) -> &[u8];
-
     /// Test if a slice is equal.
     /// The slice must have exactly LENGTH bytes.
     fn equals_known_len(&self, rhs: &[u8]) -> bool;
 }
 
-extern "C" {
+unsafe extern "C" {
     fn memcmp(s1: *const u8, s2: *const u8, n: usize) -> i32;
 }
 
-macro_rules! literal_bytes_impl {
-    ($($N:literal)+) => {
-        $(
-            impl ByteSeq for [u8; $N] {
-                const LENGTH: usize = $N;
+impl<const N: usize> ByteSeq for [u8; N] {
+    const LENGTH: usize = N;
 
-                #[inline(always)]
-                fn as_slice(&self) -> &[u8] {
-                    self
-                }
-
-                #[inline(always)]
-                fn equals_known_len(&self, rhs: &[u8]) -> bool {
-                    debug_assert!(rhs.len() == Self::LENGTH, "Slice has wrong length");
-                    if cfg!(feature = "prohibit-unsafe") {
-                        // Here's what we would like to do. However this will emit an unnecessary length compare, and an unnecessary pointer compare.
-                        self == rhs
-                    } else {
-                        // Warning: this is delicate. We intend for the compiler to emit optimized bytewise comparisons of unaligned LENGTH bytes,
-                        // where LENGTH is a compile-time constant. Rust's default == on slices will perform a pointer comparison which will always be false,
-                        // and kill any vectorization.
-                        // memcmp() will be optimized to the builtin.
-                        unsafe { memcmp(self.as_ptr(), rhs.as_ptr(), Self::LENGTH) == 0}
-                    }
-                }
-            }
-
-            impl ByteSearcher for [u8; $N] {
-                #[inline(always)]
-                fn find_in(&self, rhs: &[u8]) -> Option<usize> {
-                    if $N == 1 {
-                        return memchr::memchr(self[0], rhs)
-                    }
-                    for win in rhs.windows(Self::LENGTH) {
-                        if self.equals_known_len(win) {
-                            // Black magic?
-                            return Some((win.as_ptr() as usize) - (rhs.as_ptr() as usize));
-                        }
-                    }
-                    None
-                }
-            }
-        )+
+    #[inline(always)]
+    fn equals_known_len(&self, rhs: &[u8]) -> bool {
+        debug_assert!(rhs.len() == Self::LENGTH, "Slice has wrong length");
+        if cfg!(feature = "prohibit-unsafe") {
+            // Here's what we would like to do. However this will emit an unnecessary length compare, and an unnecessary pointer compare.
+            self == rhs
+        } else {
+            // Warning: this is delicate. We intend for the compiler to emit optimized bytewise comparisons of unaligned LENGTH bytes,
+            // where LENGTH is a compile-time constant. Rust's default == on slices will perform a pointer comparison which will always be false,
+            // and kill any vectorization.
+            // memcmp() will be optimized to the builtin.
+            unsafe { memcmp(self.as_ptr(), rhs.as_ptr(), Self::LENGTH) == 0 }
+        }
     }
 }
 
-literal_bytes_impl! {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24}
+impl ByteSearcher for [u8; 1] {
+    #[inline(always)]
+    fn find_in(&self, rhs: &[u8]) -> Option<usize> {
+        memchr::memchr(self[0], rhs)
+    }
+}
+
+impl ByteSearcher for [u8; 2] {
+    #[inline(always)]
+    fn find_in(&self, rhs: &[u8]) -> Option<usize> {
+        memchr::memchr2(self[0], self[1], rhs)
+    }
+}
+
+impl ByteSearcher for [u8; 3] {
+    #[inline(always)]
+    fn find_in(&self, rhs: &[u8]) -> Option<usize> {
+        memchr::memchr3(self[0], self[1], self[2], rhs)
+    }
+}
+
+impl ByteSearcher for memchr::memmem::Finder<'_> {
+    fn find_in(&self, rhs: &[u8]) -> Option<usize> {
+        self.find(rhs)
+    }
+}
 
 /// A ByteSet is any set of bytes.
 pub trait ByteSet {
@@ -153,7 +150,7 @@ impl SmallArraySet for [u8; 4] {
 // CharSet helper. Avoid branching in the loop to get good unrolling.
 #[allow(unused_parens)]
 #[inline(always)]
-pub fn charset_contains(set: &[char; MAX_CHAR_SET_LENGTH], c: char) -> bool {
+pub fn charset_contains(set: &[u32; MAX_CHAR_SET_LENGTH], c: u32) -> bool {
     let mut result = false;
     for &v in set.iter() {
         result |= (v == c);
@@ -232,7 +229,7 @@ impl ByteSet for AsciiBitmap {
 }
 
 /// A bitmap covering all bytes.
-#[derive(Default, Copy, Clone)]
+#[derive(Default, Copy, Clone, PartialEq, Eq)]
 #[repr(align(4))]
 pub struct ByteBitmap([u16; 16]);
 
@@ -284,10 +281,20 @@ impl ByteBitmap {
         self.0.iter().map(|v| v.count_ones()).sum()
     }
 
-    /// \return all set bytes, as a vec.
+    /// Return ourselves as an array of a fixed length.
+    /// Panics if the array is not large enough.
     #[allow(clippy::wrong_self_convention)]
-    pub fn to_vec(&self) -> Vec<u8> {
-        (0..=255).filter(|b| self.contains(*b)).collect()
+    #[inline(always)]
+    pub fn as_array<const N: usize>(&self) -> [u8; N] {
+        let mut array = [0u8; N];
+        let mut idx = 0;
+        for byte in 0..=255 {
+            if self.contains(byte) {
+                array[idx] = byte;
+                idx += 1;
+            }
+        }
+        array
     }
 
     /// \return the index of the first byte in the slice that is present in this
