@@ -5,16 +5,21 @@ use crate::bytesearch;
 use crate::cursor;
 use crate::cursor::{Backward, Direction, Forward};
 use crate::exec;
+use crate::indexing;
 use crate::indexing::{AsciiInput, ElementType, InputIndexer, Utf8Input};
-use crate::insn::{CompiledRegex, Insn, LoopFields, StartPredicate};
+#[cfg(not(feature = "utf16"))]
+use crate::insn::StartPredicate;
+use crate::insn::{CompiledRegex, Insn, LoopFields};
 use crate::matchers;
 use crate::matchers::CharProperties;
 use crate::position::PositionType;
 use crate::scm;
 use crate::scm::SingleCharMatcher;
-use crate::types::{CaptureGroupID, GroupData, LoopData, IP, MAX_CAPTURE_GROUPS};
+use crate::types::{CaptureGroupID, GroupData, IP, LoopData, LoopID, MAX_CAPTURE_GROUPS};
 use crate::util::DebugCheckIndex;
-use std::ops::Range;
+#[cfg(not(feature = "std"))]
+use alloc::vec::Vec;
+use core::ops::Range;
 
 #[derive(Clone, Debug)]
 enum BacktrackInsn<Input: InputIndexer> {
@@ -26,7 +31,7 @@ enum BacktrackInsn<Input: InputIndexer> {
     SetPosition { ip: IP, pos: Input::Position },
 
     SetLoopData {
-        id: u32,
+        id: LoopID,
         data: LoopData<Input::Position>,
     },
 
@@ -62,14 +67,14 @@ struct State<Position: PositionType> {
 }
 
 #[derive(Debug)]
-struct MatchAttempter<'a, Input: InputIndexer> {
+pub(crate) struct MatchAttempter<'a, Input: InputIndexer> {
     re: &'a CompiledRegex,
     bts: Vec<BacktrackInsn<Input>>,
     s: State<Input::Position>,
 }
 
 impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
-    fn new(re: &'a CompiledRegex, entry: Input::Position) -> Self {
+    pub(crate) fn new(re: &'a CompiledRegex, entry: Input::Position) -> Self {
         Self {
             re,
             bts: vec![BacktrackInsn::Exhausted],
@@ -191,7 +196,6 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
         let min_pos = pos;
 
         // Drive it up to the max.
-        // TODO; this is dumb.
         for _ in 0..(max - min) {
             let saved = pos;
             if !matcher.matches(input, dir, &mut pos) {
@@ -203,6 +207,174 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
         Some((min_pos, max_pos))
     }
 
+    // Compute the maximum position from a starting position, up to a limit.
+    // This is used for lazy computation in non-greedy loops.
+    fn compute_max_pos<Dir: Direction, Scm: SingleCharMatcher<Input, Dir>>(
+        input: &Input,
+        mut pos: Input::Position,
+        limit: usize,
+        dir: Dir,
+        matcher: Scm,
+    ) -> Input::Position {
+        for _ in 0..limit {
+            let saved = pos;
+            if !matcher.matches(input, dir, &mut pos) {
+                pos = saved;
+                break;
+            }
+        }
+        pos
+    }
+
+    // Helper function to extract the duplicated match blocks that handle different instruction types
+    // with different matcher functions. This significantly reduces code duplication and compile times.
+    fn with_scm_loop_impl<Dir: Direction>(
+        re: &CompiledRegex,
+        input: &Input,
+        pos: Input::Position,
+        min: usize,
+        max: usize,
+        dir: Dir,
+        ip: IP,
+    ) -> Option<(Input::Position, Input::Position)> {
+        match re.insns.iat(ip + 1) {
+            &Insn::Char(c) => {
+                let c = <<Input as InputIndexer>::Element as ElementType>::try_from(c)?;
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::Char { c })
+            }
+            &Insn::CharICase(c) => {
+                let c = <<Input as InputIndexer>::Element as ElementType>::try_from(c)?;
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::CharICase { c })
+            }
+            &Insn::Bracket(idx) => {
+                let bc = &re.brackets[idx];
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::Bracket { bc })
+            }
+            Insn::AsciiBracket(bitmap) => Self::run_scm_loop_impl(
+                input,
+                pos,
+                min,
+                max,
+                dir,
+                scm::MatchByteSet { bytes: bitmap },
+            ),
+            Insn::MatchAny => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchAny::new())
+            }
+            Insn::MatchAnyExceptLineTerminator => Self::run_scm_loop_impl(
+                input,
+                pos,
+                min,
+                max,
+                dir,
+                scm::MatchAnyExceptLineTerminator::new(),
+            ),
+            Insn::CharSet(chars) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::CharSet { chars })
+            }
+            &Insn::ByteSet2(bytes) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchByteArraySet { bytes })
+            }
+            &Insn::ByteSet3(bytes) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchByteArraySet { bytes })
+            }
+            &Insn::ByteSet4(bytes) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchByteArraySet { bytes })
+            }
+            Insn::ByteSeq1(bytes) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq2(bytes) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq3(bytes) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq4(bytes) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq5(bytes) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq6(bytes) => {
+                Self::run_scm_loop_impl(input, pos, min, max, dir, scm::MatchByteSeq { bytes })
+            }
+            _ => {
+                unreachable!("Missing SCM: {:?}", re.insns.iat(ip + 1));
+            }
+        }
+    }
+
+    // Helper function for compute_max_pos to avoid duplication
+    fn with_scm_compute_max<Dir: Direction>(
+        re: &CompiledRegex,
+        input: &Input,
+        pos: Input::Position,
+        limit: usize,
+        dir: Dir,
+        ip: IP,
+    ) -> Option<Input::Position> {
+        let result = match re.insns.iat(ip + 1) {
+            &Insn::Char(c) => {
+                let c = <<Input as InputIndexer>::Element as ElementType>::try_from(c)?;
+                Self::compute_max_pos(input, pos, limit, dir, scm::Char { c })
+            }
+            &Insn::CharICase(c) => {
+                let c = <<Input as InputIndexer>::Element as ElementType>::try_from(c)?;
+                Self::compute_max_pos(input, pos, limit, dir, scm::CharICase { c })
+            }
+            &Insn::Bracket(idx) => {
+                let bc = &re.brackets[idx];
+                Self::compute_max_pos(input, pos, limit, dir, scm::Bracket { bc })
+            }
+            Insn::AsciiBracket(bitmap) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteSet { bytes: bitmap })
+            }
+            Insn::MatchAny => Self::compute_max_pos(input, pos, limit, dir, scm::MatchAny::new()),
+            Insn::MatchAnyExceptLineTerminator => Self::compute_max_pos(
+                input,
+                pos,
+                limit,
+                dir,
+                scm::MatchAnyExceptLineTerminator::new(),
+            ),
+            Insn::CharSet(chars) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::CharSet { chars })
+            }
+            &Insn::ByteSet2(bytes) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteArraySet { bytes })
+            }
+            &Insn::ByteSet3(bytes) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteArraySet { bytes })
+            }
+            &Insn::ByteSet4(bytes) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteArraySet { bytes })
+            }
+            Insn::ByteSeq1(bytes) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq2(bytes) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq3(bytes) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq4(bytes) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq5(bytes) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteSeq { bytes })
+            }
+            Insn::ByteSeq6(bytes) => {
+                Self::compute_max_pos(input, pos, limit, dir, scm::MatchByteSeq { bytes })
+            }
+            _ => {
+                unreachable!("Missing SCM: {:?}", re.insns.iat(ip + 1));
+            }
+        };
+        Some(result)
+    }
+
     // Given that ip points at a loop whose body matches exactly one character, run
     // a "single character loop". The big idea here is that we don't need to save
     // our position every iteration: we know that our loop body matches a single
@@ -219,93 +391,25 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
         ip: IP,
         greedy: bool,
     ) -> Option<IP> {
-        // Iterate as far as we can go.
-        let loop_res = match self.re.insns.iat(ip + 1) {
-            &Insn::Char(c) => {
-                // Note this try_from may fail, for example if our char is outside ASCII.
-                // In this case we wish to not match.
-                let c = Input::Element::try_from(c)?;
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::Char { c })
-            }
-            &Insn::CharICase(c) => {
-                let c = Input::Element::try_from(c)?;
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::CharICase { c })
-            }
-            Insn::Bracket(bc) => {
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::Bracket { bc })
-            }
-            Insn::AsciiBracket(bitmap) => Self::run_scm_loop_impl(
-                input,
-                *pos,
-                min,
-                max,
-                dir,
-                scm::MatchByteSet { bytes: bitmap },
-            ),
-            Insn::MatchAny => {
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::MatchAny::new())
-            }
-            Insn::MatchAnyExceptLineTerminator => Self::run_scm_loop_impl(
-                input,
-                *pos,
-                min,
-                max,
-                dir,
-                scm::MatchAnyExceptLineTerminator::new(),
-            ),
-            Insn::CharSet(chars) => {
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::CharSet { chars })
-            }
-            &Insn::ByteSet2(bytes) => Self::run_scm_loop_impl(
-                input,
-                *pos,
-                min,
-                max,
-                dir,
-                scm::MatchByteArraySet { bytes },
-            ),
-            &Insn::ByteSet3(bytes) => Self::run_scm_loop_impl(
-                input,
-                *pos,
-                min,
-                max,
-                dir,
-                scm::MatchByteArraySet { bytes },
-            ),
-            &Insn::ByteSet4(bytes) => Self::run_scm_loop_impl(
-                input,
-                *pos,
-                min,
-                max,
-                dir,
-                scm::MatchByteArraySet { bytes },
-            ),
-            Insn::ByteSeq1(bytes) => {
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::MatchByteSeq { bytes })
-            }
-            Insn::ByteSeq2(bytes) => {
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::MatchByteSeq { bytes })
-            }
-            Insn::ByteSeq3(bytes) => {
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::MatchByteSeq { bytes })
-            }
-            Insn::ByteSeq4(bytes) => {
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::MatchByteSeq { bytes })
-            }
-            Insn::ByteSeq5(bytes) => {
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::MatchByteSeq { bytes })
-            }
-            Insn::ByteSeq6(bytes) => {
-                Self::run_scm_loop_impl(input, *pos, min, max, dir, scm::MatchByteSeq { bytes })
-            }
-            _ => {
-                // There should be no other SCMs.
-                unreachable!("Missing SCM: {:?}", self.re.insns.iat(ip + 1));
-            }
+        // For non-greedy loops, we can avoid computing the maximum match eagerly.
+        // We'll only compute it when we need to set up backtracking.
+        let (min_pos, max_pos) = if greedy {
+            // For greedy loops, compute both min and max positions
+            Self::with_scm_loop_impl(self.re, input, *pos, min, max, dir, ip)?
+        } else {
+            // For non-greedy loops, initially only compute the minimum position
+            let (min_pos, _) = Self::with_scm_loop_impl(self.re, input, *pos, min, min, dir, ip)?;
+
+            // For non-greedy loops, we only compute the max position if we need to set up backtracking
+            let max_pos = if min < max {
+                // We need to compute the max for backtracking purposes
+                Self::with_scm_compute_max(self.re, input, min_pos, max - min, dir, ip)?
+            } else {
+                min_pos
+            };
+            (min_pos, max_pos)
         };
 
-        // If loop_res is none, we failed to match at least the minimum.
-        let (min_pos, max_pos) = loop_res?;
         debug_assert!(
             if Dir::FORWARD {
                 min_pos <= max_pos
@@ -365,13 +469,13 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
         // Start with an "empty" backtrack stack.
         // TODO: consider using a stack-allocated array.
         let mut saved_bts = vec![BacktrackInsn::Exhausted];
-        std::mem::swap(&mut self.bts, &mut saved_bts);
+        core::mem::swap(&mut self.bts, &mut saved_bts);
 
         // Enter into the lookaround's instruction stream.
         let matched = self.try_at_pos(*input, ip, pos, Dir::new()).is_some();
 
         // Put back our bts.
-        std::mem::swap(&mut self.bts, &mut saved_bts);
+        core::mem::swap(&mut self.bts, &mut saved_bts);
 
         // If we are a positive lookahead that successfully matched, retain the
         // capture groups (but we need to set up backtracking). Otherwise restore
@@ -548,7 +652,8 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
 
                 match re.insns.iat(ip) {
                     &Insn::Char(c) => {
-                        let m = match Input::Element::try_from(c) {
+                        let m = match <<Input as InputIndexer>::Element as ElementType>::try_from(c)
+                        {
                             Some(c) => scm::Char { c }.matches(input, dir, &mut pos),
                             None => false,
                         };
@@ -620,7 +725,7 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
                     }
 
                     &Insn::CharICase(c) => {
-                        let m = match Input::Element::try_from(c) {
+                        let m = match <<Input as indexing::InputIndexer>::Element as indexing::ElementType>::try_from(c) {
                             Some(c) => scm::CharICase { c }.matches(input, dir, &mut pos),
                             None => false,
                         };
@@ -631,26 +736,33 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
                         scm::MatchByteSet { bytes: bitmap }.matches(input, dir, &mut pos)
                     ),
 
-                    Insn::Bracket(bc) => {
-                        next_or_bt!(scm::Bracket { bc }.matches(input, dir, &mut pos))
+                    &Insn::Bracket(idx) => {
+                        next_or_bt!(
+                            scm::Bracket {
+                                bc: &self.re.brackets[idx]
+                            }
+                            .matches(input, dir, &mut pos)
+                        )
                     }
 
                     Insn::MatchAny => {
                         next_or_bt!(scm::MatchAny::new().matches(input, dir, &mut pos))
                     }
 
-                    Insn::MatchAnyExceptLineTerminator => next_or_bt!(
-                        scm::MatchAnyExceptLineTerminator::new().matches(input, dir, &mut pos)
-                    ),
+                    Insn::MatchAnyExceptLineTerminator => {
+                        next_or_bt!(
+                            scm::MatchAnyExceptLineTerminator::new().matches(input, dir, &mut pos)
+                        )
+                    }
 
                     &Insn::WordBoundary { invert } => {
                         // Copy the positions since these destructively move them.
                         let prev_wordchar = input
                             .peek_left(pos)
-                            .map_or(false, Input::CharProps::is_word_char);
+                            .is_some_and(Input::CharProps::is_word_char);
                         let curr_wordchar = input
                             .peek_right(pos)
-                            .map_or(false, Input::CharProps::is_word_char);
+                            .is_some_and(Input::CharProps::is_word_char);
                         let is_boundary = prev_wordchar != curr_wordchar;
                         next_or_bt!(is_boundary != invert)
                     }
@@ -757,7 +869,7 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
                         next_or_bt!(matched)
                     }
 
-                    &Insn::LookaheadInsn {
+                    &Insn::Lookahead {
                         negate,
                         start_group,
                         end_group,
@@ -778,7 +890,7 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
                         }
                     }
 
-                    &Insn::LookbehindInsn {
+                    &Insn::Lookbehind {
                         negate,
                         start_group,
                         end_group,
@@ -810,7 +922,7 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
                     Insn::EnterLoop(fields) => {
                         // Entering a loop, not re-entering it.
                         self.s.loops.mat(fields.loop_id as usize).iters = 0;
-                        match self.run_loop(&fields, pos, ip) {
+                        match self.run_loop(fields, pos, ip) {
                             Some(next_ip) => {
                                 ip = next_ip;
                                 continue 'nextinsn;
@@ -849,6 +961,7 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
                             break 'backtrack;
                         }
                     }
+
                     Insn::Goal => {
                         // Keep all but the initial give-up bts.
                         self.bts.truncate(1);
@@ -888,7 +1001,14 @@ pub struct BacktrackExecutor<'r, Input: InputIndexer> {
     matcher: MatchAttempter<'r, Input>,
 }
 
+#[cfg(feature = "utf16")]
 impl<'r, Input: InputIndexer> BacktrackExecutor<'r, Input> {
+    pub(crate) fn new(input: Input, matcher: MatchAttempter<'r, Input>) -> Self {
+        Self { input, matcher }
+    }
+}
+
+impl<Input: InputIndexer> BacktrackExecutor<'_, Input> {
     fn successful_match(&mut self, start: Input::Position, end: Input::Position) -> Match {
         // We want to simultaneously map our groups to offsets, and clear the groups.
         // A for loop is the easiest way to do this while satisfying the borrow checker.
@@ -909,7 +1029,31 @@ impl<'r, Input: InputIndexer> BacktrackExecutor<'r, Input> {
         Match {
             range: self.input.pos_to_offset(start)..self.input.pos_to_offset(end),
             captures,
-            named_captures: self.matcher.re.named_group_indices.clone(),
+            group_names: self.matcher.re.group_names.clone(),
+        }
+    }
+
+    /// \return the next match for an anchored regex that only matches at the start.
+    /// This avoids any string searching and only tries matching at the given position.
+    #[cfg(not(feature = "utf16"))]
+    fn next_match_anchored(
+        &mut self,
+        pos: Input::Position,
+        next_start: &mut Option<Input::Position>,
+    ) -> Option<Match> {
+        let inp = self.input;
+        // For anchored regexes, only try matching at the current position
+        if let Some(end) = self.matcher.try_at_pos(inp, 0, pos, Forward::new()) {
+            // If we matched the empty string, we have to increment.
+            if end != pos {
+                *next_start = Some(end)
+            } else {
+                *next_start = inp.next_right_pos(end);
+            }
+            Some(self.successful_match(pos, end))
+        } else {
+            // Anchored regex failed to match at this position, no more matches
+            None
         }
     }
 
@@ -924,7 +1068,11 @@ impl<'r, Input: InputIndexer> BacktrackExecutor<'r, Input> {
         let inp = self.input;
         loop {
             // Find the next start location, or None if none.
-            pos = inp.find_bytes(pos, prefix_search)?;
+            // Don't try this unless CODE_UNITS_ARE_BYTES - i.e. don't do byte searches
+            // on UTF-16 or UCS2.
+            if Input::CODE_UNITS_ARE_BYTES {
+                pos = inp.find_bytes(pos, prefix_search)?;
+            }
             if let Some(end) = self.matcher.try_at_pos(inp, 0, pos, Forward::new()) {
                 // If we matched the empty string, we have to increment.
                 if end != pos {
@@ -940,7 +1088,7 @@ impl<'r, Input: InputIndexer> BacktrackExecutor<'r, Input> {
     }
 }
 
-impl<'a, Input: InputIndexer> exec::MatchProducer for BacktrackExecutor<'a, Input> {
+impl<Input: InputIndexer> exec::MatchProducer for BacktrackExecutor<'_, Input> {
     type Position = Input::Position;
 
     fn initial_position(&self, offset: usize) -> Option<Self::Position> {
@@ -952,27 +1100,28 @@ impl<'a, Input: InputIndexer> exec::MatchProducer for BacktrackExecutor<'a, Inpu
         pos: Input::Position,
         next_start: &mut Option<Input::Position>,
     ) -> Option<Match> {
+        // When UTF-16 support is active prefix search is not used due to the different encoding.
+        #[cfg(feature = "utf16")]
+        return self.next_match_with_prefix_search(pos, next_start, &bytesearch::EmptyString {});
+
+        #[cfg(not(feature = "utf16"))]
         match &self.matcher.re.start_pred {
             StartPredicate::Arbitrary => {
                 self.next_match_with_prefix_search(pos, next_start, &bytesearch::EmptyString {})
             }
-            StartPredicate::ByteSeq1(bytes) => {
+            StartPredicate::StartAnchored => self.next_match_anchored(pos, next_start),
+            StartPredicate::ByteSet1(bytes) => {
                 self.next_match_with_prefix_search(pos, next_start, bytes)
             }
-            StartPredicate::ByteSeq2(bytes) => {
+            StartPredicate::ByteSet2(bytes) => {
                 self.next_match_with_prefix_search(pos, next_start, bytes)
             }
-            StartPredicate::ByteSeq3(bytes) => {
+            StartPredicate::ByteSet3(bytes) => {
                 self.next_match_with_prefix_search(pos, next_start, bytes)
             }
-            StartPredicate::ByteSeq4(bytes) => {
-                self.next_match_with_prefix_search(pos, next_start, bytes)
+            StartPredicate::ByteSeq(bytes) => {
+                self.next_match_with_prefix_search(pos, next_start, bytes.as_ref())
             }
-            &StartPredicate::ByteSet2(bytes) => self.next_match_with_prefix_search(
-                pos,
-                next_start,
-                &bytesearch::ByteArraySet(bytes),
-            ),
             StartPredicate::ByteBracket(bitmap) => {
                 self.next_match_with_prefix_search(pos, next_start, bitmap)
             }
@@ -984,7 +1133,7 @@ impl<'r, 't> exec::Executor<'r, 't> for BacktrackExecutor<'r, Utf8Input<'t>> {
     type AsAscii = BacktrackExecutor<'r, AsciiInput<'t>>;
 
     fn new(re: &'r CompiledRegex, text: &'t str) -> Self {
-        let input = Utf8Input::new(text);
+        let input = Utf8Input::new(text, re.flags.unicode);
         Self {
             input,
             matcher: MatchAttempter::new(re, input.left_end()),
