@@ -1,10 +1,16 @@
 //! Bytecode instructions for a compiled regex
 
-use std::collections::HashMap;
+#[cfg(not(feature = "std"))]
+use {
+    alloc::{string::String, vec::Vec},
+    hashbrown::HashMap,
+};
 
 use crate::api;
 use crate::bytesearch::{AsciiBitmap, ByteArraySet, ByteBitmap};
-use crate::types::{BracketContents, CaptureGroupID};
+use crate::types::{BracketContents, CaptureGroupID, LoopID};
+extern crate memchr;
+use memchr::memmem;
 
 type JumpTarget = u32;
 
@@ -21,7 +27,7 @@ pub const MAX_CHAR_SET_LENGTH: usize = 4;
 
 #[derive(Debug, Clone)]
 pub struct LoopFields {
-    pub loop_id: u32,
+    pub loop_id: LoopID,
     pub min_iters: usize,
     pub max_iters: usize,
     pub greedy: bool,
@@ -35,10 +41,10 @@ pub enum Insn {
     Goal,
 
     /// Match a single char.
-    Char(char),
+    Char(u32),
 
     /// Match a single char, case-insensitive.
-    CharICase(char),
+    CharICase(u32),
 
     /// Match the start of a line (if multiline); emitted by '^'
     StartOfLine,
@@ -92,16 +98,15 @@ pub enum Insn {
     /// Perform a backreference match.
     BackRef(u32),
 
-    /// Match the next character against a bracket.
-    /// TODO: this is a very heavyweight instruction, consider breaking it up.
-    Bracket(BracketContents),
+    /// Match the next character against the bracket contents, stored at the given index in the CompiledRegex.
+    Bracket(usize),
 
     /// A simple bitmap bracket for ASCII.
     /// It contains a bitmap of the range [0, 127].
     AsciiBracket(AsciiBitmap),
 
     /// Perform a lookahead assertion.
-    LookaheadInsn {
+    Lookahead {
         negate: bool,
         start_group: CaptureGroupID,
         end_group: CaptureGroupID,
@@ -109,7 +114,7 @@ pub enum Insn {
     },
 
     /// Perform a lookbehind assertion.
-    LookbehindInsn {
+    Lookbehind {
         negate: bool,
         start_group: CaptureGroupID,
         end_group: CaptureGroupID,
@@ -123,7 +128,7 @@ pub enum Insn {
 
     /// Match any of the contained chars
     /// There is no length field; characters are simply duplicated as necessary.
-    CharSet([char; MAX_CHAR_SET_LENGTH]),
+    CharSet([u32; MAX_CHAR_SET_LENGTH]),
 
     /// Match the next byte against some possibilities.
     ByteSet2(ByteArraySet<[u8; 2]>),
@@ -155,30 +160,50 @@ pub enum Insn {
 
 /// The peeled prefix start predicate.
 /// This is a fast way of locating the first potential match.
-#[derive(Debug, Copy, Clone)]
+#[derive(Debug, Clone)]
 pub enum StartPredicate {
     /// May match an arbitrary sequence.
     Arbitrary,
 
-    /// Look for literal bytes.
-    ByteSeq1([u8; 1]),
-    ByteSeq2([u8; 2]),
-    ByteSeq3([u8; 3]),
-    ByteSeq4([u8; 4]),
-
-    /// Look for any of the contained bytes.
+    /// Look for the first instance of any of the given bytes.
+    ByteSet1([u8; 1]),
     ByteSet2([u8; 2]),
+    ByteSet3([u8; 3]),
+
+    /// Look for a byte sequence.
+    ByteSeq(Box<memmem::Finder<'static>>),
 
     /// Look for a byte which matches the bitmap.
     ByteBracket(ByteBitmap),
+
+    /// The regex is anchored to the start of the line/string.
+    /// This avoids string searching entirely and only tries matching at the beginning.
+    StartAnchored,
 }
 
 #[derive(Debug, Clone)]
 pub struct CompiledRegex {
+    // Sequence of instructions.
     pub insns: Vec<Insn>,
+
+    // The bracket contents, indexed by the value of the `Bracket` instruction.
+    pub brackets: Vec<BracketContents>,
+
+    // Predicate to rapidly find the first potential match.
     pub start_pred: StartPredicate,
+
+    // Number of loops, used to populate loop data.
     pub loops: u32,
+
+    // Number of capture groups, used to populate capture group data.
     pub groups: u32,
-    pub named_group_indices: HashMap<String, u16>,
+
+    // A list of capture group names. This is either:
+    //   - Empty, if there were no named capture groups.
+    //   - A list of names with length `groups`, corresponding to the capture
+    //     group names in order. Groups without names have an empty string.
+    pub group_names: Box<[Box<str>]>,
+
+    // Flags controlling matching.
     pub flags: api::Flags,
 }
