@@ -1,45 +1,38 @@
-use crate::cursor;
 use crate::cursor::Direction;
 use crate::indexing::{ElementType, InputIndexer};
 use crate::types::BracketContents;
-use crate::unicode;
+use crate::{cursor, unicode};
 
 pub trait CharProperties {
     type Element: ElementType;
 
     /// Case-fold an element.
-    fn fold(c: Self::Element) -> Self::Element;
-
-    /// \return whether these two elements fold to the same value.
-    fn fold_equals(c1: Self::Element, c2: Self::Element) -> bool {
-        c1 == c2 || Self::fold(c1) == Self::fold(c2)
-    }
+    fn fold(c: Self::Element, unicode: bool) -> Self::Element;
 
     /// \return whether this is a word char.
     /// ES9 21.2.2.6.2.
     fn is_word_char(c: Self::Element) -> bool {
-        match c.as_char() {
-            'a'..='z' => true,
-            'A'..='Z' => true,
-            '0'..='9' => true,
-            '_' => true,
-            _ => false,
-        }
+        let c = c.as_u32();
+        'a' as u32 <= c && c <= 'z' as u32
+            || 'A' as u32 <= c && c <= 'Z' as u32
+            || '0' as u32 <= c && c <= '9' as u32
+            || c == '_' as u32
     }
 
     /// ES9 11.3
     fn is_line_terminator(c: Self::Element) -> bool {
-        let c = c.as_char();
-        c == '\u{000A}' || c == '\u{000D}' || c == '\u{2028}' || c == '\u{2029}'
+        matches!(c.as_u32(), 0x000A | 0x000D | 0x2028 | 0x2029)
     }
 
     /// \return whether the bracket \p bc matches the given character \p c,
     /// respecting case. Respects 'invert'.
     #[inline(always)]
-    fn bracket(bc: &BracketContents, c: Self::Element) -> bool {
-        let cp = c.into();
-        let contained = bc.cps.intervals().iter().any(|r| r.contains(cp));
-        contained ^ bc.invert
+    fn bracket(bc: &BracketContents, cp: Self::Element) -> bool {
+        let cp = cp.into();
+        if bc.cps.contains(cp) {
+            return !bc.invert;
+        }
+        bc.invert
     }
 }
 
@@ -48,8 +41,8 @@ pub struct UTF8CharProperties {}
 impl CharProperties for UTF8CharProperties {
     type Element = char;
 
-    fn fold(c: Self::Element) -> Self::Element {
-        unicode::fold(c)
+    fn fold(c: Self::Element, unicode: bool) -> Self::Element {
+        char::from_u32(unicode::fold_code_point(c.into(), unicode)).unwrap_or(c)
     }
 }
 
@@ -57,8 +50,20 @@ pub struct ASCIICharProperties {}
 impl CharProperties for ASCIICharProperties {
     type Element = u8;
 
-    fn fold(c: Self::Element) -> Self::Element {
-        c.to_ascii_lowercase()
+    fn fold(c: Self::Element, _: bool) -> Self::Element {
+        c.to_ascii_uppercase()
+    }
+}
+
+#[cfg(feature = "utf16")]
+pub struct Utf16CharProperties {}
+
+#[cfg(feature = "utf16")]
+impl CharProperties for Utf16CharProperties {
+    type Element = u32;
+
+    fn fold(c: Self::Element, unicode: bool) -> Self::Element {
+        unicode::fold_code_point(c, unicode)
     }
 }
 
@@ -66,16 +71,16 @@ impl CharProperties for ASCIICharProperties {
 pub fn backref<Input: InputIndexer, Dir: Direction>(
     input: &Input,
     dir: Dir,
-    orig_range: std::ops::Range<Input::Position>,
+    orig_range: core::ops::Range<Input::Position>,
     pos: &mut Input::Position,
 ) -> bool {
-    cursor::subrange_eq(input, dir, pos, orig_range.start, orig_range.end)
+    input.subrange_eq(dir, pos, orig_range)
 }
 
 pub fn backref_icase<Input: InputIndexer, Dir: Direction>(
     input: &Input,
     dir: Dir,
-    orig_range: std::ops::Range<Input::Position>,
+    orig_range: core::ops::Range<Input::Position>,
     pos: &mut Input::Position,
 ) -> bool {
     let ref_input = input.subinput(orig_range);
@@ -87,7 +92,7 @@ pub fn backref_icase<Input: InputIndexer, Dir: Direction>(
     while let Some(c1) = cursor::next(&ref_input, dir, &mut ref_pos) {
         let mut matched = false;
         if let Some(c2) = cursor::next(input, dir, pos) {
-            matched = Input::CharProps::fold_equals(c1, c2)
+            matched = input.fold_equals(c1, c2)
         }
         if !matched {
             return false;
