@@ -6,7 +6,7 @@ use crate::cursor;
 use crate::cursor::{Backward, Direction, Forward};
 use crate::exec;
 use crate::indexing::{AsciiInput, ElementType, InputIndexer, Utf8Input};
-use crate::insn::{CompiledRegex, Insn, LoopFields};
+use crate::insn::{CompiledRegex, Insn, LoopFields, StartPredicate};
 use crate::matchers;
 use crate::matchers::CharProperties;
 use crate::position::PositionType;
@@ -14,8 +14,9 @@ use crate::scm;
 use crate::scm::SingleCharMatcher;
 use crate::types::{GroupData, LoopData};
 use crate::util::DebugCheckIndex;
-use std::collections::HashMap;
-use std::ops::Range;
+#[cfg(not(feature = "std"))]
+use alloc::{string::String, vec::Vec};
+use core::ops::Range;
 
 #[derive(Debug, Clone)]
 struct State<Position: PositionType> {
@@ -113,22 +114,18 @@ fn try_match_state<Input: InputIndexer, Dir: Direction>(
         Insn::Goal => StateMatch::Complete,
         Insn::JustFail => StateMatch::Fail,
         &Insn::Char(c) => match cursor::next(input, dir, &mut s.pos) {
-            Some(c2) => nextinsn_or_fail!(c == c2.as_char()),
+            Some(c2) => nextinsn_or_fail!(c == c2.as_u32()),
             _ => StateMatch::Fail,
         },
-        &Insn::CharICase(c) => {
-            let c = match Input::Element::try_from(c) {
-                Some(c) => c,
-                None => return StateMatch::Fail,
-            };
-            match cursor::next(input, dir, &mut s.pos) {
-                Some(c2) => nextinsn_or_fail!(c == c2 || Input::CharProps::fold(c2) == c),
-                _ => StateMatch::Fail,
+        &Insn::CharICase(c) => match cursor::next(input, dir, &mut s.pos) {
+            Some(c2) => {
+                nextinsn_or_fail!(c == c2.as_u32() || input.fold(c2).as_u32() == c)
             }
-        }
+            _ => StateMatch::Fail,
+        },
 
         Insn::CharSet(v) => match cursor::next(input, dir, &mut s.pos) {
-            Some(c) => nextinsn_or_fail!(charset_contains(v, c.as_char())),
+            Some(c) => nextinsn_or_fail!(charset_contains(v, c.as_u32())),
             _ => StateMatch::Fail,
         },
 
@@ -192,10 +189,10 @@ fn try_match_state<Input: InputIndexer, Dir: Direction>(
         &Insn::BeginCaptureGroup(group_idx) => {
             let group = &mut s.groups[group_idx as usize];
             if Dir::FORWARD {
-                std::debug_assert!(!group.start_matched(), "Group should not have been entered");
+                core::debug_assert!(!group.start_matched(), "Group should not have been entered");
                 group.start = Some(s.pos);
             } else {
-                std::debug_assert!(!group.end_matched(), "Group should not have been entered");
+                core::debug_assert!(!group.end_matched(), "Group should not have been entered");
                 group.end = Some(s.pos);
             }
             nextinsn_or_fail!(true)
@@ -204,13 +201,13 @@ fn try_match_state<Input: InputIndexer, Dir: Direction>(
         &Insn::EndCaptureGroup(group_idx) => {
             let group = &mut s.groups[group_idx as usize];
             if Dir::FORWARD {
-                std::debug_assert!(group.start_matched(), "Group should have been entered");
+                core::debug_assert!(group.start_matched(), "Group should have been entered");
                 group.end = Some(s.pos);
             } else {
-                std::debug_assert!(group.end_matched(), "Group should have been exited");
+                core::debug_assert!(group.end_matched(), "Group should have been exited");
                 group.start = Some(s.pos);
             }
-            std::debug_assert!(
+            core::debug_assert!(
                 group.end >= group.start,
                 "Exit pos should be after start pos"
             );
@@ -239,7 +236,7 @@ fn try_match_state<Input: InputIndexer, Dir: Direction>(
             nextinsn_or_fail!(matched)
         }
 
-        &Insn::LookaheadInsn {
+        &Insn::Lookahead {
             negate,
             start_group: _,
             end_group: _,
@@ -260,7 +257,7 @@ fn try_match_state<Input: InputIndexer, Dir: Direction>(
             }
         }
 
-        &Insn::LookbehindInsn {
+        &Insn::Lookbehind {
             negate,
             start_group: _,
             end_group: _,
@@ -283,13 +280,13 @@ fn try_match_state<Input: InputIndexer, Dir: Direction>(
         &Insn::LoopAgain { begin } => {
             s.ip = begin as usize;
             match re.insns.iat(s.ip) {
-                Insn::EnterLoop(ref lf) => run_loop(s, &lf, false),
+                Insn::EnterLoop(lf) => run_loop(s, lf, false),
                 _ => panic!("LoopAgain does not point at EnterLoop"),
             }
         }
         Insn::Loop1CharBody { .. } => panic!("Loop1CharBody unimplemented for pikevm"),
-        Insn::Bracket(bc) => match cursor::next(input, dir, &mut s.pos) {
-            Some(c) => nextinsn_or_fail!(Input::CharProps::bracket(bc, c)),
+        &Insn::Bracket(idx) => match cursor::next(input, dir, &mut s.pos) {
+            Some(c) => nextinsn_or_fail!(Input::CharProps::bracket(&re.brackets[idx], c)),
             _ => StateMatch::Fail,
         },
 
@@ -309,10 +306,10 @@ fn try_match_state<Input: InputIndexer, Dir: Direction>(
         &Insn::WordBoundary { invert } => {
             let prev_wordchar = input
                 .peek_left(s.pos)
-                .map_or(false, Input::CharProps::is_word_char);
+                .is_some_and(Input::CharProps::is_word_char);
             let curr_wordchar = input
                 .peek_right(s.pos)
-                .map_or(false, Input::CharProps::is_word_char);
+                .is_some_and(Input::CharProps::is_word_char);
             let is_boundary = prev_wordchar != curr_wordchar;
             nextinsn_or_fail!(is_boundary != invert)
         }
@@ -323,7 +320,7 @@ fn successful_match<Input: InputIndexer>(
     input: Input,
     start: Input::Position,
     state: &State<Input::Position>,
-    named_captures: HashMap<String, u16>,
+    group_names: Box<[Box<str>]>,
 ) -> Match {
     let group_to_offset = |mr: &GroupData<Input::Position>| -> Option<Range<usize>> {
         mr.as_range().map(|r| Range {
@@ -335,7 +332,7 @@ fn successful_match<Input: InputIndexer>(
     Match {
         range: input.pos_to_offset(start)..input.pos_to_offset(state.pos),
         captures,
-        named_captures,
+        group_names,
     }
 }
 
@@ -370,7 +367,7 @@ impl<'a, Input: InputIndexer> MatchAttempter<'a, Input> {
                 StateMatch::Continue => {}
                 StateMatch::Complete => {
                     // Give the successful state to the caller.
-                    std::mem::swap(init_state, s);
+                    core::mem::swap(init_state, s);
                     self.states.clear();
                     return true;
                 }
@@ -391,7 +388,7 @@ impl<'r, 't> exec::Executor<'r, 't> for PikeVMExecutor<'r, Utf8Input<'t>> {
     type AsAscii = PikeVMExecutor<'r, AsciiInput<'t>>;
 
     fn new(re: &'r CompiledRegex, text: &'t str) -> Self {
-        let input = Utf8Input::new(text);
+        let input = Utf8Input::new(text, re.flags.unicode);
         Self {
             input,
             matcher: MatchAttempter::new(re),
@@ -411,7 +408,7 @@ impl<'r, 't> exec::Executor<'r, 't> for PikeVMExecutor<'r, AsciiInput<'t>> {
     }
 }
 
-impl<'a, Input: InputIndexer> exec::MatchProducer for PikeVMExecutor<'a, Input> {
+impl<Input: InputIndexer> exec::MatchProducer for PikeVMExecutor<'_, Input> {
     type Position = Input::Position;
 
     fn initial_position(&self, offset: usize) -> Option<Self::Position> {
@@ -424,6 +421,37 @@ impl<'a, Input: InputIndexer> exec::MatchProducer for PikeVMExecutor<'a, Input>
         next_start: &mut Option<Self::Position>,
     ) -> Option<Match> {
         let re = self.matcher.re;
+
+        // Check if this is an anchored regex - if so, only try matching at the current position
+        if matches!(re.start_pred, StartPredicate::StartAnchored) {
+            let mut state = State {
+                pos,
+                ip: 0,
+                loops: vec![LoopData::new(pos); re.loops as usize],
+                groups: vec![GroupData::new(); re.groups as usize],
+            };
+            if self
+                .matcher
+                .try_at_pos(self.input, &mut state, Forward::new())
+            {
+                let end = state.pos;
+                if end != pos {
+                    *next_start = Some(end)
+                } else {
+                    *next_start = self.input.next_right_pos(end)
+                }
+                return Some(successful_match(
+                    self.input,
+                    pos,
+                    &state,
+                    re.group_names.clone(),
+                ));
+            }
+            // Anchored regex failed to match at this position
+            return None;
+        }
+
+        // Standard matching - try at each position
         // Note the "initial" loop position is ignored. Use whatever is most convenient.
         let mut state = State {
             pos,
@@ -447,7 +475,7 @@ impl<'a, Input: InputIndexer> exec::MatchProducer for PikeVMExecutor<'a, Input>
                     self.input,
                     start,
                     &state,
-                    re.named_group_indices.clone(),
+                    re.group_names.clone(),
                 ));
             }
             match self.input.next_right_pos(start) {
