@@ -1,44 +1,38 @@
+//! Support for quickly finding potential match locations.
 use crate::bytesearch::ByteBitmap;
 use crate::codepointset;
 use crate::insn::StartPredicate;
 use crate::ir;
 use crate::ir::Node;
-use crate::util::utf8_first_byte;
-use std::cmp;
-use std::convert::TryInto;
+use crate::util::{add_utf8_first_bytes_to_bitmap, utf8_first_byte};
+#[cfg(not(feature = "std"))]
+use alloc::{boxed::Box, vec::Vec};
+use memchr::memmem;
 
-/// Support for quickly finding potential match locations.
+/// Check if a node is anchored to the start of the line/string.
+/// Returns true if the node begins with a StartOfLine anchor.
+fn is_start_anchored(n: &Node) -> bool {
+    match n {
+        Node::Anchor(ir::AnchorType::StartOfLine) => true,
+        Node::Cat(nodes) => {
+            // For concatenation, check if the first node is start-anchored
+            nodes.first().is_some_and(is_start_anchored)
+        }
+        Node::CaptureGroup(child, ..) | Node::NamedCaptureGroup(child, ..) => {
+            is_start_anchored(child)
+        }
+        // Other nodes are not anchored
+        _ => false,
+    }
+}
 
 /// Convert the code point set to a first-byte bitmap.
 /// That is, make a list of all of the possible first bytes of every contained
 /// code point, and store that in a bitmap.
 fn cps_to_first_byte_bitmap(input: &codepointset::CodePointSet) -> Box<ByteBitmap> {
-    // Rather than walking over all contained codepoints, which could be very
-    // expensive, we perform searches over all bytes.
-    // Note that UTF-8 first-bytes increase monotone and contiguously.
-    // Increasing a code point will either increase the first byte by 1 or 0.
-    let mut bitmap = Box::new(ByteBitmap::default());
-    let mut ivs = input.intervals();
-    for target_byte in 0..=255 {
-        let search = ivs.binary_search_by(|iv| {
-            let left_byte = utf8_first_byte(iv.first);
-            let right_byte = utf8_first_byte(iv.last);
-            debug_assert!(left_byte <= right_byte);
-            if left_byte <= target_byte && target_byte <= right_byte {
-                cmp::Ordering::Equal
-            } else if left_byte > target_byte {
-                cmp::Ordering::Greater
-            } else {
-                debug_assert!(right_byte < target_byte, "Binary search logic is wrong");
-                cmp::Ordering::Less
-            }
-        });
-        if let Ok(idx) = search {
-            bitmap.set(target_byte);
-            // Since our bytes are strictly increasing, previous intervals will never hold
-            // the next byte. Help out our binary search by removing previous intervals.
-            ivs = &ivs[idx..];
-        }
+    let mut bitmap = Box::<ByteBitmap>::default();
+    for iv in input.intervals() {
+        add_utf8_first_bytes_to_bitmap(*iv, &mut bitmap);
     }
     bitmap
 }
@@ -100,15 +94,14 @@ impl AbstractStartPredicate {
             Self::Arbitrary => StartPredicate::Arbitrary,
             Self::Sequence(vals) => match vals.len() {
                 0 => StartPredicate::Arbitrary,
-                1 => StartPredicate::ByteSeq1(vals[..].try_into().unwrap()),
-                2 => StartPredicate::ByteSeq2(vals[..].try_into().unwrap()),
-                3 => StartPredicate::ByteSeq3(vals[..].try_into().unwrap()),
-                _ => StartPredicate::ByteSeq4(vals[..4].try_into().unwrap()),
+                1 => StartPredicate::ByteSet1([vals[0]]),
+                _ => StartPredicate::ByteSeq(Box::new(memmem::Finder::new(&vals).into_owned())),
             },
             Self::Set(bm) => match bm.count_bits() {
                 0 => StartPredicate::Arbitrary,
-                1 => StartPredicate::ByteSeq1(bm.to_vec()[..].try_into().unwrap()),
-                2 => StartPredicate::ByteSet2(bm.to_vec()[..].try_into().unwrap()),
+                1 => StartPredicate::ByteSet1(bm.as_array()),
+                2 => StartPredicate::ByteSet2(bm.as_array()),
+                3 => StartPredicate::ByteSet3(bm.as_array()),
                 _ => StartPredicate::ByteBracket(*bm),
             },
         }
@@ -135,7 +128,7 @@ fn compute_start_predicate(n: &Node) -> Option<AbstractStartPredicate> {
             // Pick the first bytes out.
             let bytes = chars
                 .iter()
-                .map(|&c| utf8_first_byte(c as u32))
+                .map(|&c| utf8_first_byte(c))
                 .collect::<Vec<_>>();
             Some(AbstractStartPredicate::Set(Box::new(ByteBitmap::new(
                 &bytes,
@@ -144,7 +137,7 @@ fn compute_start_predicate(n: &Node) -> Option<AbstractStartPredicate> {
 
         // We assume that most char nodes have been optimized to ByteSeq or AnyBytes2, so skip
         // these.
-        // TODO: we could support icase through bitmap of defolded first bytes.
+        // TODO: we could support icase through bitmap of de-folded first bytes.
         Node::Char { .. } => arbitrary,
 
         // Cats return the first non-None value, if any.
@@ -209,7 +202,7 @@ fn compute_start_predicate(n: &Node) -> Option<AbstractStartPredicate> {
             } else {
                 &bc.cps
             };
-            let bitmap = cps_to_first_byte_bitmap(&cps);
+            let bitmap = cps_to_first_byte_bitmap(cps);
             Some(AbstractStartPredicate::Set(bitmap))
         }
     }
@@ -217,6 +210,14 @@ fn compute_start_predicate(n: &Node) -> Option<AbstractStartPredicate> {
 
 /// \return the start predicate for a Regex.
 pub fn predicate_for_re(re: &ir::Regex) -> StartPredicate {
+    // Check if the regex is anchored to the start - if so, we can optimize
+    // by avoiding string searching entirely. However, only do this when
+    // multiline mode is disabled, since in multiline mode ^ can match
+    // at the beginning of any line, not just the string start.
+    if is_start_anchored(&re.node) && !re.flags.multiline {
+        return StartPredicate::StartAnchored;
+    }
+
     compute_start_predicate(&re.node)
         .unwrap_or(AbstractStartPredicate::Arbitrary)
         .resolve_to_insn()
