@@ -1,5 +1,8 @@
+#![allow(clippy::uninlined_format_args)]
+
+#[track_caller]
 fn test_1_error(pattern: &str, expected_err: &str) {
-    let res = regress::Regex::new(pattern);
+    let res = regress::Regex::with_flags(pattern, "u");
     assert!(res.is_err(), "Pattern should not have parsed: {}", pattern);
 
     let err = res.err().unwrap().text;
@@ -26,21 +29,26 @@ fn test_excessive_capture_groups() {
 
 #[test]
 fn test_syntax_errors() {
-    test_1_error(r"*", "Nothing to repeat");
-    test_1_error(r"x**", "Nothing to repeat");
-    test_1_error(r"?", "Nothing to repeat");
-    test_1_error(r"{3,5}", "Nothing to repeat");
+    test_1_error(r"*", "Invalid atom character");
+    test_1_error(r"x**", "Invalid atom character");
+    test_1_error(r"?", "Invalid atom character");
+    test_1_error(r"{3,5}", "Invalid atom character");
     test_1_error(r"x{5,3}", "Invalid quantifier");
 
-    test_1_error(r"]", "Unbalanced bracket");
+    test_1_error(r"]", "Invalid atom character");
     test_1_error(r"[abc", "Unbalanced bracket");
 
     test_1_error(r"(", "Unbalanced parenthesis");
     test_1_error(r"(?!", "Unbalanced parenthesis");
     test_1_error(r"abc)", "Unbalanced parenthesis");
 
-    test_1_error(r"[z-a]", "Invalid character range");
-    test_1_error(r"[\d-z]", "Invalid character range");
+    test_1_error(
+        r"[z-a]",
+        "Range values reversed, start char code is greater than end char code.",
+    );
+
+    // In unicode mode this is not allowed.
+    test_1_error(r"[a-\s]", "Invalid character range");
 
     test_1_error("\\", "Incomplete escape");
 
